from typing import List

from discord import Bot, Guild, Role, Member

from dbotbase.bot_config import BotConfig
from dbotbase.discord_utils import get_users, get_roles


class BaseBot(Bot):
    def __init__(self, config: BotConfig, *args, **options):
        super().__init__(*args, **options)
        self.config = config

    @property
    def server_id(self) -> int:
        return self.config.server_id

    @property
    def token(self) -> str:
        return self.config.bot_token

    @property
    def prefix(self) -> str:
        return self.config.bot_prefix

    @property
    def admin_role_names(self) -> List[str]:
        return self.config.bot_admin_role_names

    def get_admin_roles(self, guild: Guild) -> List[Role]:
        return get_roles(guild=guild, role_names=self.admin_role_names)

    @property
    def admin_ids(self) -> List[int]:
        return self.config.bot_admin_ids

    def get_admin_users(self, guild: Guild) -> List[Member]:
        return get_users(guild=guild, user_ids=self.admin_ids)
